#include "ehs/io/audio/BaseAudioDevice.h"

namespace ehs
{
	BaseAudioDevice::BaseAudioDevice()
		: type(AudioDeviceType::ALL), dataType(DataType::SINT_8), byteDepth(0), sampleRate(0), channels(0),
		period(20000), latency(150000), maxFrames(0)
    {
    }

	BaseAudioDevice::BaseAudioDevice(const BaseAudioDevice& device)
		: type(device.type), dataType(device.dataType), byteDepth(device.byteDepth), sampleRate(device.sampleRate),
		channels(device.channels), period(device.period), latency(device.latency), maxFrames(0)
    {
    }

	BaseAudioDevice& BaseAudioDevice::operator=(const BaseAudioDevice& device)
    {
        if (this == &device)
            return *this;

		type = device.type;
		dataType = device.dataType;
		byteDepth = device.byteDepth;
		sampleRate = device.sampleRate;
		channels = device.channels;
		period = device.period;
		latency = device.latency;
		maxFrames = device.maxFrames;

        return *this;
    }

	void BaseAudioDevice::OpenStream()
	{
	}

	void BaseAudioDevice::CloseStream()
	{
	}

	UInt_64 BaseAudioDevice::SendStream(const void *data, const UInt_64 size)
	{
		return 0;
	}

	UInt_64 BaseAudioDevice::ReceiveStream(void *data, const UInt_64 size)
	{
		return 0;
	}

	void BaseAudioDevice::BridgeStreams(const UInt_64 bufferSize)
	{
		Byte* buffer = new Byte[bufferSize];

		UInt_64 result = ReceiveStream(buffer, bufferSize);
		while (result < bufferSize)
			result += ReceiveStream(&buffer[result], bufferSize - result);

		result -= SendStream(buffer, result);
		while (result)
			result -= SendStream(&buffer[bufferSize - result], result);

		delete[] buffer;
	}

	AudioDeviceType BaseAudioDevice::GetType() const
    {
        return type;
    }

	void BaseAudioDevice::SetDataType(const DataType newDataType)
	{
		if (IsStreaming())
			return;

		dataType = newDataType;
		byteDepth = ToByteDepth(newDataType);
	}

	DataType BaseAudioDevice::GetDataType() const
	{
		return dataType;
	}

	UInt_8 BaseAudioDevice::GetByteDepth() const
	{
		return byteDepth;
	}

	UInt_16 BaseAudioDevice::GetBitDepth() const
	{
		return byteDepth * 8;
	}

	void BaseAudioDevice::SetSampleRate(const UInt_32 newSampleRate)
	{
		if (IsStreaming())
			return;

		sampleRate = newSampleRate;
	}

	UInt_32 BaseAudioDevice::GetSampleRate() const
	{
		return sampleRate;
	}

	void BaseAudioDevice::SetChannels(const UInt_32 newChannels)
	{
		if (IsStreaming())
			return;

		channels = newChannels;
	}

	UInt_16 BaseAudioDevice::GetChannels() const
	{
		return channels;
	}

	UInt_32 BaseAudioDevice::GetFrameSize() const
	{
		return byteDepth * channels;
	}

	void BaseAudioDevice::SetPeriod(const UInt_32 newPeriod)
	{
		if (IsStreaming())
			return;

		period = newPeriod;
	}

	UInt_32 BaseAudioDevice::GetPeriod() const
	{
		return period;
	}

	void BaseAudioDevice::SetLatency(const UInt_32 newLatency)
	{
		if (IsStreaming())
			return;

		latency = newLatency;
	}

	UInt_32 BaseAudioDevice::GetLatency() const
	{
		return latency;
	}

	UInt_64 BaseAudioDevice::GetMaxFrames() const
	{
		return maxFrames;
	}

	bool BaseAudioDevice::IsStreaming() const
	{
		return false;
	}

	bool BaseAudioDevice::IsValid() const
	{
		return false;
	}

	BaseAudioDevice BaseAudioDevice::GetDefault(const AudioDeviceType type)
	{
		return {};
	}

	Array<BaseAudioDevice> BaseAudioDevice::Get(const AudioDeviceType type, const AudioDeviceState state)
	{
		return {};
	}
}