#include "ehs/system/Mutex_W32.h"
#include "ehs/Str.h"
#include "ehs/Log.h"

namespace ehs
{
	Mutex::~Mutex()
	{
		if (!initialized)
			return;

		if (!CloseHandle(hdl))
			EHS_LOG_INT(LogType::ERR, 0, "Failed to uninitialize mutex with error #" + Str_8::FromNum(GetLastError()) + ".");
	}

	Mutex::Mutex()
		: hdl(nullptr)
	{
	}

	Mutex::Mutex(const Mutex& mutex)
		: BaseMutex(mutex), hdl(nullptr)
	{
	}

	Mutex& Mutex::operator=(const Mutex& mutex)
	{
		if (this == &mutex)
			return *this;

		BaseMutex::operator=(mutex);

		hdl = nullptr;

		return *this;
	}

	void Mutex::Initialize()
	{
		if (initialized)
			return;

		hdl = CreateMutexW(nullptr, FALSE, nullptr);
		if (!hdl)
		{
			EHS_LOG_INT(LogType::ERR, 0, "Failed to create mutex with error #" + Str_8::FromNum(GetLastError()) + ".");
			return;
		}

		initialized = true;
	}

	void Mutex::UnInitialize()
	{
		if (!initialized)
			return;

		if (!CloseHandle(hdl))
			EHS_LOG_INT(LogType::ERR, 0, "Failed to uninitialize mutex with error #" + Str_8::FromNum(GetLastError()) + ".");

		initialized = false;
	}

	void Mutex::Lock()
	{
		if (locked)
			return;

		if (WaitForSingleObject(hdl, EHS_INFINITE) == WAIT_FAILED)
		{
			EHS_LOG_INT(LogType::ERR, 0, "Failed to lock mutex with error #" + Str_8::FromNum(GetLastError()) + ".");
			return;
		}

		locked = true;
	}

	void Mutex::Unlock()
	{
		if (!locked)
			return;

		if (!ReleaseMutex(hdl))
			EHS_LOG_INT(LogType::ERR, 0, "Failed to unlock mutex with error #" + Str_8::FromNum(GetLastError()) + ".");

		locked = false;
	}
}